/* Recent.m created by shayman on Mon 23-Mar-1998 */

#import "Recent.h"
#import "NotifyingSavePanel.h"

@interface NSObject (NSDocumentControllerAvoidWarning)
+ (id)sharedDocumentController;
- (id)openDocumentWithContentsOfFile:(NSString *)fileName display:(BOOL)display;
@end

@implementation Recent
+ (void)initialize
{
    [[NotifyingSavePanel class] poseAsClass:[NSSavePanel class]];
}


- separator
{
#ifdef WIN32
    /* NT cannot handle unicode, sigh */
    return @"  --  ";
#else
#define EM_DASH 0x2014  /* Unicode em dash */
    static NSString *separatorString = nil;

    /* NSWindow uses an em dash as a separator; looks better than two dashes */
    if (separatorString == nil) {
        unichar sepchars[5] = {
            ' ', ' ', EM_DASH, ' ', ' '
        };
        separatorString = [[NSString alloc] initWithCharacters:sepchars length:5];
    }
    return separatorString;
#endif
}

- defaultName
{
    return [NSString stringWithFormat:@"RecentMenu%@",
        [dict objectForKey:@"name"]];
}

- (void) initMenu
{
    NSMenuItem *newItem;
    id docMenu;
    id title= [NSString stringWithFormat:@"Recent %@", [dict objectForKey:@"name"]];
#ifdef WIN32
    int docMenuIndex = 0;
#else
    int docMenuIndex = 1;
#endif

    docMenu = [[[[NSApp mainMenu] itemArray] objectAtIndex:docMenuIndex] target];
    menu = [[NSMenu alloc] initWithTitle:title];
    newItem = [docMenu insertItemWithTitle:title
                                    action:@selector(whatever:)
                             keyEquivalent:@""
                                   atIndex:2];
    [docMenu setSubmenu:menu forItem:newItem];
}

- (void)setupRecentMenu
{
    id menuItem, enu, aModel;
    id recentModels;
    recentModels = [[[NSUserDefaults standardUserDefaults] objectForKey:[self defaultName]] mutableCopy];

    // clear out the existing menu

    while (  [[menu itemArray] count] ) {
        [menu removeItem: [[menu itemArray]  lastObject]];
    }

    // build a new list of recent menu items
    enu = [recentModels objectEnumerator];
    while ( aModel = [enu nextObject] ) {
        id title = [NSString stringWithFormat:@"%@%@%@",
            [aModel lastPathComponent],
            [self separator],
            [aModel stringByDeletingLastPathComponent]];

        menuItem = [menu addItemWithTitle:title
                        action:@selector(openMe:)
                keyEquivalent:@""];
        [menuItem setTarget:self];
    }


}
- (int) keep
{
    id keepString = [dict objectForKey:@"keep"];
    if ( keepString ) {
        return [keepString intValue];
    }
    return 10;
}
- (void) addToRecentList:(NSString *)filename
{
    id recentList = [[[NSUserDefaults standardUserDefaults] objectForKey:[self defaultName]] mutableCopy];
    if ( ! recentList )
        recentList = [NSMutableArray array];
    // Remove this doc if already on the list
    [recentList removeObject:filename];
    // and add it at the beginning
    [recentList insertObject:filename atIndex:0];

    // and remove old ones beyond the 10th (or however many the "keep" prop says)

    while ( [recentList count] > [self keep] )
        [recentList removeLastObject];

    [[NSUserDefaults standardUserDefaults] setObject:recentList forKey:[self defaultName]];

    [self setupRecentMenu];
}



// When you run as an input manager, it appears that exactly one instance
// of the class is alloc'd/init'd.

- init
{
    id notes, note, enu;

    props = [NSDictionary dictionaryWithContentsOfFile:
        [[NSBundle bundleForClass:[self class]]
                  pathForResource:@"Props" ofType:@"plist"]];
    [props retain];

    // Is this application one we're interested in?
    dict = [props objectForKey:[[NSProcessInfo processInfo] processName]];
    if ( !dict ) {
        //NSLog(@"RecentMenu - not available for %@ apps", [[NSProcessInfo processInfo] processName]);
        return nil;
    }
        
    notes = [NSArray arrayWithObjects:
        	NSWindowDidBecomeMainNotification,
                NSWindowDidBecomeKeyNotification,
                NSWindowDidResignMainNotification,
                NSWindowDidResignKeyNotification,
        nil];

    enu = [notes objectEnumerator];
    while ( note = [enu nextObject] ) {
    

        [[NSNotificationCenter defaultCenter]
                    addObserver:self
                    selector:@selector(noteWindow:)
                        name:note
                        object:nil];
    }
    [[NSNotificationCenter defaultCenter]
        addObserver:self
           selector:@selector(saveOk:)
               name:@"SavePanelSaysOK"
             object:nil];
    
    [self initMenu];
    [self setupRecentMenu];
    return [super init];
}

    
    
// Has this window got a represented filename?  if so, let's remember it.
- (void)rememberWindow:(NSWindow *)win
{
    NSString * filename = [win representedFilename];
    id mustContain;

    if ( !filename || ([filename length]) == 0 )
        return;

    // hack - skip windows named UNTITLED or Untitled, even if they
    // are represented filenames (which happens in Draw and WOB)
    if ( [[[filename lastPathComponent] uppercaseString] hasPrefix:@"UNTITLED"]  )
        return;

    // special hack for PB - and perhaps some other apps - look for
    // a file with a particular name inside the document.  in PB's case,
    // the windows have a represented filename of the name of the project dir,
    // and that dir needs to contain PB.project.  this hack is necessary
    // because PB can also display windows for all sorts of other files,
    // and we're only interested in the PB.project ones.

    if ( mustContain = [dict objectForKey:@"mustContain"] ) {
        id path = [filename stringByAppendingPathComponent:mustContain];
        if ( ! [[NSFileManager defaultManager] fileExistsAtPath:path] )
            return;
    }

    // ok - we like this file
    [self addToRecentList:filename];
    
    
}

- (void)noteWindow:(NSNotification *)note
{
    [self rememberWindow:[note object]];
}
// special message we get after being notified that the open or save panel has
// done something.  good time to review the name of the current window - which
// won't change until the save panel has done its thing and whatever asked
// it to save something does its thing - so don't check the win name
// right away

- (void)saveOk:sender
{
    [self performSelector:@selector(rememberWindow:)
               withObject:[NSApp mainWindow]
               afterDelay:1];
}

/*
 * The Recent Whatever menu items send this message; figure out what filename
 * they were referring to, and ask the app's delegate to open it.
 * Which is the only portable way I could think of to make this work
 * in any app; the application:openFile; method is the one an app has
 * to implement if it wants to receive files double-clicked-upon.
 * I think.
 */
- (void) openMe:sender
{
    id path;
    id parts = [[sender title] componentsSeparatedByString:[self separator]];
    path = [[parts lastObject]
                stringByAppendingPathComponent:[parts objectAtIndex:0]];
    // [[NSWorkspace sharedWorkspace] openFile:path];
    if ([[NSApp delegate] respondsToSelector:@selector(application:openFile:)]) {
        [[NSApp delegate] application:NSApp openFile:path];
    }
    else
    {
        Class docControllerClass = NSClassFromString(@"NSDocumentController");
        id docController = [docControllerClass sharedDocumentController];
        [docController openDocumentWithContentsOfFile:path display:YES];
    }
}

@end
